﻿using System;
using System.Xml;
using System.IO;
using System.Windows.Forms;
using System.Globalization;

namespace SaintsRowFov
{
    
    public partial class MainForm : Form
    {
        //----------------------------------------------------------------------
        //      MEMBERS
        //----------------------------------------------------------------------

        private enum GameVersion { None, SR3, SR4, GOOH };
        private string _defaultXtbl;
        private int _defaultFov;




        //----------------------------------------------------------------------
        //      METHODS
        //----------------------------------------------------------------------

        public MainForm()
        {
            InitializeComponent();
        }

        private GameVersion GetGameVersion()
        {
            #if DEBUG
                return GameVersion.SR3;
            #else
            if (File.Exists("SaintsRowTheThird.exe"))
                return GameVersion.SR3;
            else if (File.Exists("SaintsRowIV.exe"))
                return GameVersion.SR4;
            else if (File.Exists("SaintsRowGatOutOfHell.exe"))
                return GameVersion.GOOH;
            else
                return GameVersion.None;
            #endif
        }

        private XmlDocument LoadXtblResourceFile(out XmlNodeList fovNodes)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(this.GetType().Assembly.GetManifestResourceStream(_defaultXtbl));
            fovNodes = doc.GetElementsByTagName("base_fov");
            return doc;
        }

        private void InitializeProgram()
        {
            XmlNodeList fovNodes;
            XmlDocument doc = LoadXtblResourceFile(out fovNodes);

            //set the defaul FOV and slider min
            DefaultFovLabel.Text = fovNodes[0].InnerText;
            _defaultFov = Convert.ToInt32(fovNodes[0].InnerText);
            FovSlider.Minimum = _defaultFov;

            //get the current user fov
            if (File.Exists("camera_free.xtbl"))
            {
                doc.Load("camera_free.xtbl");
                fovNodes = doc.GetElementsByTagName("base_fov");

                //set the slider value and label text
                FovSlider.Value = Convert.ToInt32(fovNodes[0].InnerText);
                CurrentFovLabel.Text = fovNodes[0].InnerText;
            }
            else
            {
                //set the slider value and label text
                FovSlider.Value = _defaultFov;
                CurrentFovLabel.Text = _defaultFov.ToString();
            }
        }



        //----------------------------------------------------------------------
        //      MESSAGE HANDLERS
        //----------------------------------------------------------------------

        private void MainForm_Load(object sender, EventArgs e)
        {
            switch (GetGameVersion())
            {
                case GameVersion.SR3:
                    LogoImage.Image = Properties.Resources.sr3_logo;
                    _defaultXtbl = "SaintsRowFov.Resources.sr3_default.xtbl";
                    InitializeProgram();
                    break;

                case GameVersion.SR4:
                    LogoImage.Image = Properties.Resources.sr4_logo;
                    _defaultXtbl = "SaintsRowFov.Resources.sr4_default.xtbl";
                    InitializeProgram();
                    break;

                case GameVersion.GOOH:
                    LogoImage.Image = Properties.Resources.gooh_logo;
                    _defaultXtbl = "SaintsRowFov.Resources.gooh_default.xtbl";
                    InitializeProgram();
                    break;

                default:
                    MessageBox.Show("No Valid Saints Row game Detected.\n\nEnsure that this program is located in the same folder as your Saints Row .exe file", "Saints Row FOV");
                    Application.Exit();
                    break;
            }

        }			
		
        private void FovSlider_Scroll(object sender, EventArgs e)
        {
            CurrentFovLabel.Text = FovSlider.Value.ToString();
            ApplyButton.Enabled = true;
        }

        private void ApplyButton_Click(object sender, EventArgs e)
        {
            //how much to scale non-default FOV's by?
            float scalar = (float)FovSlider.Value / (float)_defaultFov;

            //user set FOV back to default?
            if (scalar <= 1.0f)
            {
                //delete the modded file if it exists
                if (File.Exists("camera_free.xtbl"))
                    File.Delete("camera_free.xtbl");

                ApplyButton.Enabled = false;
                return;
            }

            //reload the default xtbl file
            XmlNodeList fovNodes;
            XmlDocument doc = LoadXtblResourceFile(out fovNodes);

            //now modify the FOV xml tags in it
            foreach (XmlNode node in fovNodes)
            {
                float value = (float)Convert.ToDouble(node.InnerText, CultureInfo.InvariantCulture);
                if (value == _defaultFov)
                    node.InnerText = FovSlider.Value.ToString();
                else
                    node.InnerText = (value * scalar).ToString(CultureInfo.InvariantCulture);
            }

            //now save it to disk
            doc.Save("camera_free.xtbl");
            ApplyButton.Enabled = false;
        }
    }
}



